/*********************************************************************
*                SEGGER Microcontroller GmbH & Co. KG                *
*                        The Embedded Experts                        *
**********************************************************************
*                                                                    *
*       (c) 1995 - 2016 SEGGER Microcontroller GmbH & Co. KG         *
*                                                                    *
*     Internet: segger.com   Support: support_embos@segger.com       *
*                                                                    *
**********************************************************************
*                                                                    *
*       embOS * Real time operating system for microcontrollers      *
*                                                                    *
*                                                                    *
*       Please note:                                                 *
*                                                                    *
*       Knowledge of this file may under no circumstances            *
*       be used to write a similar product or a real-time            *
*       operating system for in-house use.                           *
*                                                                    *
*       Thank you for your fairness !                                *
*                                                                    *
**********************************************************************
*                                                                    *
*       OS version: 4.16                                             *
*                                                                    *
**********************************************************************

----------------------------------------------------------------------
File    : BSP.c
Purpose : BSP for Freescale DEMOEM eval board
--------  END-OF-HEADER  ---------------------------------------------
*/

#include "BSP.h"
#include "RTOS.h"

/*********************************************************************
*
*       Defines
*
**********************************************************************
*/

/****** Addresses of sfrs and peripheral identifiers ****************/
#define PTEDD   (*(volatile OS_U8*) (0xFFFF8101))  // Port E Data Direction Register
#define PTESET  (*(volatile OS_U8*) (0xFFFF8102))  // Port E Data Set Register
#define PTECLR  (*(volatile OS_U8*) (0xFFFF8103))  // Port E Data Clear Register
#define PTETOG  (*(volatile OS_U8*) (0xFFFF8104))  // Port E Data Toggle Register

/*********************************************************************
*
*       Global functions
*
**********************************************************************
*/

/*********************************************************************
*
*       BSP_Init()
*/
void BSP_Init(void) {
  PTEDD  = 0x30;  // Leds at Pin4 and Pin5
  PTESET = 0x30;
}

/*********************************************************************
*
*       LED switching routines
*       LEDs are switched on with low level on port lines
*/
void BSP_SetLED(int Index) {
  PTESET = (OS_U8)(1 << (Index + 4));           
}

void BSP_ClrLED(int Index) {
  PTECLR = (OS_U8)(1 << (Index + 4));           
}

void BSP_ToggleLED(int Index) {
  PTETOG = (OS_U8)(1 << (Index + 4));           
}

/****** EOF *********************************************************/


