/*********************************************************************
*                SEGGER Microcontroller GmbH & Co. KG                *
*                        The Embedded Experts                        *
**********************************************************************
*                                                                    *
*       (c) 1995 - 2016 SEGGER Microcontroller GmbH & Co. KG         *
*                                                                    *
*     Internet: segger.com   Support: support_embos@segger.com       *
*                                                                    *
**********************************************************************
*                                                                    *
*       embOS * Real time operating system for microcontrollers      *
*                                                                    *
*                                                                    *
*       Please note:                                                 *
*                                                                    *
*       Knowledge of this file may under no circumstances            *
*       be used to write a similar product or a real-time            *
*       operating system for in-house use.                           *
*                                                                    *
*       Thank you for your fairness !                                *
*                                                                    *
**********************************************************************
*                                                                    *
*       OS version: 4.16                                             *
*                                                                    *
**********************************************************************

----------------------------------------------------------------------
File    : RTOSINIT_MC51EM256.c
          for Freescale V1 (MC51EM256)

Purpose : Initializes and handles the hardware for the OS as far
          as required by the OS.
          Feel free to modify this file acc. to your
          target system.

--------  END-OF-HEADER  ---------------------------------------------
*/

#include "RTOS.h"

/*********************************************************************
*
*       Configuration
*
**********************************************************************
*/

#define ALLOW_NESTED_INTERRUPTS  (1)  // Caution: Nesting interrupts will cause higher stack load

/*********************************************************************
*
*       Clock frequency settings
*/
#define OS_FSYS (48000000L)

#ifndef   OS_TICK_FREQ
  #define OS_TICK_FREQ (1000)
#endif

#ifndef   OS_PCLK_TIMER
  #define OS_PCLK_TIMER  (OS_FSYS/2)
#endif

/*********************************************************************
*
*       Configuration of communication to OSView
*/
#ifndef   OS_VIEW_ENABLE            // Global enable of communication
  #define OS_VIEW_ENABLE    (0)     // Default: off, uart is not tested because DEMOEM eval board
#endif                              // has no dsub9 connector

/*********************************************************************
*
*       UART settings for OSView
*       If you do not want (or can not due to hardware limitations)
*       to dedicate a UART to OSView, please define it to be -1
*       Currently UART0 and UART1 are supported and the standard
*       setup enables UART 0 per default
*/
#ifndef   OS_UART
  #define OS_UART (0)
#endif

#ifndef   OS_BAUDRATE
  #define OS_BAUDRATE (38400)
#endif

#ifndef   OS_PCLK_UART
  #define OS_PCLK_UART  (OS_FSYS/2)
#endif

/*********************************************************************
*
* Configure interrupt vector table location
*/
#ifndef    OS_ISRHANDLERTABLE_ROM
  #define  OS_ISRHANDLERTABLE_ROM (0)
#endif                    

/****** End of configuration settings *******************************/

/*********************************************************************
*
*       UART settings for OSView
*
*       Automatically generated from configuration settings
*/
#define OS_UART_USED   (OS_VIEW_ENABLE && ((OS_UART == 0) || (OS_UART == 1)))

/*********************************************************************
*
*       Local defines (sfrs used in RTOSInit.c)
*
**********************************************************************
*/
#define _NUM_INT_SOURCES           50

#define SOPT1      (*(volatile OS_U8*) (0xFFFF8281))  // System Options register
#define IRTC_CTRL  (*(volatile OS_U16*)(0xFFFF8650))  // RTC control register
#define IRTC_ISR   (*(volatile OS_U16*)(0xFFFF8654))  // RTC status register
#define IRTC_IER   (*(volatile OS_U16*)(0xFFFF8656))  // RTC interrupt enable register
#define SCGC4      (*(volatile OS_U8*) (0xFFFF8289))  // Status and Control register

#define TPMSC      (*(volatile OS_U8*) (0xFFFF8600))  // Timer status and Control register
#define TPMCNT     (*(volatile OS_U16*)(0xFFFF8602))  // Timer counter register 
#define TPMMOD     (*(volatile OS_U16*)(0xFFFF8603))  // Timer counter modulo register
#define TPMC1SC    (*(volatile OS_U8*) (0xFFFF8605))  // Timer channel 0 status and control register 
#define TPMC1V     (*(volatile OS_U16*)(0xFFFF8606))  // Timer channel 0 value register

#define OS_TPM_INT_ID 5

#define SCI1BDH    (*(volatile OS_U8*) (0xFFFF82C0))  // Baud Rate register High 
#define SCI1BDL    (*(volatile OS_U8*) (0xFFFF82C1))  // Baud Rate register Low
#define SCI1C1     (*(volatile OS_U8*) (0xFFFF82C2))  // Control register 1
#define SCI1C2     (*(volatile OS_U8*) (0xFFFF82C3))  // Control register 1
#define SCI1S1     (*(volatile OS_U8*) (0xFFFF82C4))  // Status register 1
#define SCI1S2     (*(volatile OS_U8*) (0xFFFF82C5))  // Status register 2
#define SCI1C3     (*(volatile OS_U8*) (0xFFFF82C6))  // Control register 3
#define SCI1D      (*(volatile OS_U8*) (0xFFFF82C7))  // Data register

#define SCI2BDH    (*(volatile OS_U8*) (0xFFFF82E0))  // Baud Rate register High 
#define SCI2BDL    (*(volatile OS_U8*) (0xFFFF82E1))  // Baud Rate register Low
#define SCI2C1     (*(volatile OS_U8*) (0xFFFF82E2))  // Control register 1
#define SCI2C2     (*(volatile OS_U8*) (0xFFFF82E3))  // Control register 1
#define SCI2S1     (*(volatile OS_U8*) (0xFFFF82E4))  // Status register 1
#define SCI2S2     (*(volatile OS_U8*) (0xFFFF82E5))  // Status register 2
#define SCI2C3     (*(volatile OS_U8*) (0xFFFF82E6))  // Control register 3
#define SCI2D      (*(volatile OS_U8*) (0xFFFF82E7))  // Data register

#define ICSC1      (*(volatile OS_U8*) (0xFFFF8220))
#define ICSC2      (*(volatile OS_U8*) (0xFFFF8221))
#define ICSTRM     (*(volatile OS_U8*) (0xFFFF8222))
#define ICSSC      (*(volatile OS_U8*) (0xFFFF8223))

#if (OS_UART == 0) 
  #define SCIxBDH    SCI1BDH 
  #define SCIxBDL    SCI1BDL 
  #define SCIxC1     SCI1C1
  #define SCIxC2     SCI1C2      
  #define SCIxS1     SCI1S1 
  #define SCIxS2     SCI1S2
  #define SCIxC3     SCI1C3
  #define SCIxD      SCI1D 
  #define SCx_RX_VECTORNUMBER  20
  #define SCx_TX_VECTORNUMBER  21
#elif (OS_UART == 1) 
  #define SCIxBDH    SCI2BDH 
  #define SCIxBDL    SCI2BDL 
  #define SCIxC1     SCI2C1
  #define SCIxC2     SCI2C2      
  #define SCIxS1     SCI2S1 
  #define SCIxS2     SCI2S2
  #define SCIxC3     SCI2C3
  #define SCIxD      SCI2D 
  #define SCx_RX_VECTORNUMBER  24
  #define SCx_TX_VECTORNUMBER  25
#endif

#define TPMC1SC_CH0F             (1 << 7)
        
#define SCIxS1_ERRORS_MASK       0x0F  
#define SCIxS1_RDRF              (1 << 5)
#define SCIxS1_TDRE              (1 << 7)
#define SCIxC2_TE                (1 << 3)
#define SCIxC2_RE                (1 << 2)
#define SCIxC2_RIE               (1 << 5)
#define SCIxC2_TIE               (1 << 6)

/*********************************************************************
*
*       Prototypes
*
**********************************************************************
*/
static void _OS_IrqDefaultHandler(void);
static void _OS_SystemIrqhandler(void);
#if OS_UART_USED
void OS_COM_ISR_Rx(void);
void OS_COM_ISR_Tx(void);
#endif /* OS_UART_USED */

  
/*********************************************************************
*
*       Externals
*
**********************************************************************
*/
         
/*********************************************************************
*
*       Static data
*
**********************************************************************
*/
#if (OS_ISRHANDLERTABLE_ROM != 0)
static OS_ISR_HANDLER* const _papISR[_NUM_INT_SOURCES] = 
#else
static OS_ISR_HANDLER* _papISR[_NUM_INT_SOURCES] = 
#endif
{
  _OS_IrqDefaultHandler,                         // Handler for interrupt 64 
  _OS_IrqDefaultHandler,                         // Handler for interrupt 65 
  _OS_IrqDefaultHandler,                         // Handler for interrupt 66  
  _OS_IrqDefaultHandler,                         // Handler for interrupt 67  
  _OS_IrqDefaultHandler,                         // Handler for interrupt 68  
  _OS_SystemIrqhandler,                          // Handler for interrupt 69  
  _OS_IrqDefaultHandler,                         // Handler for interrupt 70  
  _OS_IrqDefaultHandler,                         // Handler for interrupt 71  
  _OS_IrqDefaultHandler,                         // Handler for interrupt 72  
  _OS_IrqDefaultHandler,                         // Handler for interrupt 73  
  _OS_IrqDefaultHandler,                         // Handler for interrupt 74  
  _OS_IrqDefaultHandler,                         // Handler for interrupt 75  
  _OS_IrqDefaultHandler,                         // Handler for interrupt 76  
  _OS_IrqDefaultHandler,                         // Handler for interrupt 77  
  _OS_IrqDefaultHandler,                         // Handler for interrupt 78  
  _OS_IrqDefaultHandler,                         // Handler for interrupt 79  
  _OS_IrqDefaultHandler,                         // Handler for interrupt 80  
  _OS_IrqDefaultHandler,                         // Handler for interrupt 81  
  _OS_IrqDefaultHandler,                         // Handler for interrupt 82  
  _OS_IrqDefaultHandler,                         // Handler for interrupt 83  
#if (OS_UART_USED && (OS_UART == 0))
  OS_COM_ISR_Rx,                                 // Handler for interrupt 84 UART0 Rx
  OS_COM_ISR_Tx,                                 // Handler for interrupt 85 UART0 Tx 
#else
  _OS_IrqDefaultHandler,                         // Handler for interrupt 84 UART0 Rx
  _OS_IrqDefaultHandler,                         // Handler for interrupt 85 UART0 Tx 
#endif  
  _OS_IrqDefaultHandler,                         // Handler for interrupt 86  
  _OS_IrqDefaultHandler,                         // Handler for interrupt 87  

#if (OS_UART_USED && (OS_UART == 1))
  OS_COM_ISR_Rx,                                 // Handler for interrupt 88 UART1 Rx
  OS_COM_ISR_Tx,                                 // Handler for interrupt 89 UART1 Tx
#else
  _OS_IrqDefaultHandler,                         // Handler for interrupt 88 UART1 Rx
  _OS_IrqDefaultHandler,                         // Handler for interrupt 89 UART1 Tx
#endif   
  _OS_IrqDefaultHandler,                         // Handler for interrupt 90  
  _OS_IrqDefaultHandler,                         // Handler for interrupt 91  
  _OS_IrqDefaultHandler,                         // Handler for interrupt 92  
  _OS_IrqDefaultHandler,                         // Handler for interrupt 93  
  _OS_IrqDefaultHandler,                         // Handler for interrupt 94  
  _OS_IrqDefaultHandler,                         // Handler for interrupt 95  
  _OS_IrqDefaultHandler,                         // Handler for interrupt 96  
  _OS_IrqDefaultHandler,                         // Handler for interrupt 97  
  _OS_IrqDefaultHandler,                         // Handler for interrupt 98  
  _OS_IrqDefaultHandler,                         // Handler for interrupt 99  
  _OS_IrqDefaultHandler,                         // Handler for interrupt 100  
  _OS_IrqDefaultHandler,                         // Handler for interrupt 101  
  _OS_IrqDefaultHandler,                         // Handler for interrupt 102   
};

/*********************************************************************
*
*       Local functions
*
**********************************************************************
*/

/*********************************************************************
*
*       _OS_GetHWTimerCycles()
*
* Function description
*   Returns the current hardware timer count value
*
* Return value
*   Current timer count value
*/
static unsigned int _OS_GetHWTimerCycles(void) {
  return ((unsigned int)(TPMCNT));
}

/*********************************************************************
*
*       _OS_GetHWTimer_IntPending()
*
* Function description
*   Returns if the hardware timer interrupt pending flag is set
*
* Return value
*   == 0; Interrupt pending flag not set
*   != 0: Interrupt pending flag set
*/
static unsigned int _OS_GetHWTimer_IntPending(void) {
  return ((unsigned int)(TPMC1SC & TPMC1SC_CH0F));
}


/*********************************************************************
*
*       Global functions
*
**********************************************************************
*/

/*********************************************************************
*
*       OS_InitHW()
*
*       Initialize the hardware (timer) required for the OS to run.
*       May be modified, if an other timer should be used
*/
#define OS_TIMER_RELOAD (OS_PCLK_TIMER/OS_TICK_FREQ)
#if (OS_TIMER_RELOAD >= 0x10000)
  #error "TPM1 timer can not be used, please check configuration"
#endif

void OS_InitHW(void) {
  int i;
  
  //
  // Structure with information about timer frequency, tick frequency, etc.
  // for micro second precise system time.
  // SysTimerConfig.TimerFreq will be set later, thus it is initialized with zero.
  //
  OS_SYSTIMER_CONFIG SysTimerConfig = {OS_FSYS, OS_TICK_FREQ, 0, _OS_GetHWTimerCycles, _OS_GetHWTimer_IntPending};
  //
  // Setup values for usec precise system time functions
  //
  OS_Config_SysTimer(&SysTimerConfig);

  OS_IncDI();
  //
  // Intilialize interrupt vector table in RAM if required
  //
#if (OS_ISRHANDLERTABLE_ROM == 0)
  for (i = 0; i < _NUM_INT_SOURCES; +i++) {
    _papISR[i] = (OS_ISR_HANDLER*)_OS_IrqDefaultHandler;
  }
#endif  
  //
  //  Disable Watchdog 
  //
  SOPT1    &= ~0x0C;
  //
  //  Disable RTC, first disable write protect by writing 0-1-3-2 in control register
  //
  IRTC_CTRL = 0;
  IRTC_CTRL = 1;
  IRTC_CTRL = 3;
  IRTC_CTRL = 2;
  IRTC_IER  = 0x00;
  IRTC_ISR  = 0x00;
  //
  // Init PLL, use external RC oscillator (4 MHz x 1536 / 128 = 48 MHz)
  //
  ICSC1  = 0x12;                 // Initialization of the ICS control register 1
  ICSC2  = 0x22;                 // Initialization of the ICS control register 2
  ICSSC &= ~0x60;                // Initialization of the ICS status and control
  ICSSC |= 0x80;                 // Initialization of the ICS status and control
  while((ICSSC & 0xC0) != 0x80); // Wait until the FLL switches to Low range DCO mode   
  //
  // Initialize embOS timer
  //
  OS_EnableISR(OS_TPM_INT_ID, _OS_SystemIrqhandler);  
  //SCGC4   |= 0x10;               // Starting timer clock
  TPMSC    = 0x00;               // Stop HW; disable overflow interrupt and set prescaler to 0 
  TPMC1SC  = 0x50;               // Set output compare mode and enable compare interrupt 
  TPMC1V   = (OS_TIMER_RELOAD);  // Initialize appropriate value to the compare/modulo/reload register      
  TPMMOD   = (OS_TIMER_RELOAD);
  TPMCNT   = 0x00;               // Reset HW Counter  
  TPMSC    = 0x08;               // No prescaler, select BusClk and run timer  
  OS_COM_Init();
  OS_DecRI();
}

/*********************************************************************
*
*       Idle loop  (OS_Idle)
*
*       Please note:
*       This is basically the "core" of the idle loop.
*       This core loop can be changed, but:
*       The idle loop does not have a stack of its own, therefore no
*       functionality should be implemented that relies on the stack
*       to be preserved. However, a simple program loop can be programmed
*       (like toggeling an output or incrementing a counter)
*/
void OS_Idle(void) {           // Idle loop: No task is ready to exec 
  while (1) {
  }
}
  
/*********************************************************************
*
*        _OS_SystemIrqhandler
*/
void _OS_SystemIrqhandler(void) {
  TPMC1SC &= ~TPMC1SC_CH0F;  // Reset compare interrupt request flag, TPM1C0SC: CH0F=0  
  OS_TICK_Handle();  
}

/*********************************************************************
*
*       Get time [cycles]
*
*       This routine is required for task-info via OSView or high
*       resolution time maesurement functions.
*       It returns the system time in timer clock cycles.
*/
OS_U32 OS_GetTime_Cycles(void) {
  OS_U32 Time;
  OS_U32 Cnt;
  
  Time = (OS_U32)OS_GetTime32();
  Cnt  = TPMCNT;
  //  
  // Check if timer interrupt pending ...
  // 
  if (TPMC1SC & TPMC1SC_CH0F) {  // Adjust result
    Cnt = TPMCNT;
    Time++;
  }
  return (OS_TIMER_RELOAD * Time) + Cnt;
}

/*********************************************************************
*
*       OS_ConvertCycles2us
*
*       Convert Cycles into micro seconds.
*
*       If your clock frequency is not a multiple of 1 MHz,
*       you may have to modify this routine in order to get proper
*       diagonstics.
*
*       This routine is required for profiling or high resolution time
*       measurement only. It does not affect operation of the OS.
*/
OS_U32 OS_ConvertCycles2us(OS_U32 Cycles) {
  return Cycles/(OS_PCLK_TIMER/1000000);
}

/*********************************************************************
*
*       Communication for OSView via UART (optional)
*
**********************************************************************
*/

#if OS_UART_USED

#define BAUDDIVIDE (OS_PCLK_UART / (16 * OS_BAUDRATE))

/*********************************************************************
*
*       OS_COM_ISR_Rx() 
*
*       OS USART Rx interrupt handler 
*/
void OS_COM_ISR_Rx(void) {
  OS_U8 c;
  OS_U8 SciStat;
  
  SciStat = SCIxS1;  // Read current status
  c       = SCIxD;   // Read data, reset status
  
  if ((SciStat & SCIxS1_ERRORS_MASK) == 0)  {
    if (SciStat & SCIxS1_RDRF) {  // No Error occured and Rx buffer full?
      OS_OnRx(c);
    }
  }
}

/*********************************************************************
*
*       OS_COM_ISR_Tx() 
*
*       OS USART Tx interrupt handler 
*/
void OS_COM_ISR_Tx(void) {
  if (OS_OnTx()) {             // No more characters to send?  
    SCIxC2 &= ~SCIxC2_TIE;     // Disable Transmission Complete Interrupt Enable (for TC)     
  }
}

/*********************************************************************
*
*       OS_COM_Send1()
*       Never call this function directly from your application
*/
void OS_COM_Send1(OS_U8 c) {
  while ((SCIxS1 & SCIxS1_TDRE) == 0);  // Is transmit buffer empty?
  SCIxC2 |= SCIxC2_TIE; 
  SCIxD   = c;  
}

/*********************************************************************
*
*       OS_COM_Init()
*       Initialize UART for OSView
*/
void OS_COM_Init(void) {       // Initialize UART, enable UART interrupts 
  OS_EnableISR(SCx_RX_VECTORNUMBER, OS_COM_ISR_Rx);
  OS_EnableISR(SCx_TX_VECTORNUMBER, OS_COM_ISR_Tx);
  SCIxC1  = 0x00;              // SCI Control Register 1
  SCIxBDH = BAUDDIVIDE >> 8;   // Set baudrate
  SCIxBDL = BAUDDIVIDE & 0xFF;
  SCIxC2  =   SCIxC2_RIE       // Receive Complete Interrupt Enable (for TC)
            | SCIxC2_TE        // Transmitter Enable
            | SCIxC2_RE;       // Receiver Enable
}

#else 

void OS_COM_Init(void) {}
void OS_COM_Send1(OS_U8 c) {
  (void) c;                    // Avoid compiler warning 
  OS_COM_ClearTxActive();      // Let the OS know that Tx is not busy 
}

#endif /*  OS_UART_USED  */

/****** Final check of configuration ********************************/

#ifndef OS_UART_USED
  #error "OS_UART_USED has to be defined"
#endif

/*********************************************************************
*
*       OS interrupt handler and ISR specific functions
*
**********************************************************************
*/

/*********************************************************************
*
*       OS_IrqDefaultHandler
*
*/
void _OS_IrqDefaultHandler(void) {
#if OS_DEBUG
  OS_DI();
  while (1) {
  //
  // The NOPs are necessary for bdm debugger,
  // otherwise the bdm debugger locks up
  //
  asm(NOP);
  asm(NOP);
  }
#endif
}

/*********************************************************************
*
*       OS_irq_handler
*
*       Detect reason for IRQ and call correspondig service routine.
*       OS_irq_handler is called from OS_ISR_Handler function
*       found in RTOS.s
*       We arrive here with interrupts disabled
*/
static char n;
void OS_irq_handler(int Status) {
  int ISRIndex;

  ISRIndex = ((Status >> 18) & 0xFF) - 64;  // Extract ISR Index
#if OS_DEBUG
  if ((ISRIndex >= _NUM_INT_SOURCES) || (ISRIndex < 0)) {
    OS_Error(OS_ERR_ISR_INDEX);
    return;
  }
#endif
#if ALLOW_NESTED_INTERRUPTS
  OS_EnterNestableInterrupt();         // Now interrupts may be reenabled. If nesting should be allowed
#else
  OS_EnterInterrupt();                 // Inform OS that interrupt handler is running
#endif
  _papISR[ISRIndex]();                 // Call installed handler
#if ALLOW_NESTED_INTERRUPTS
  OS_LeaveNestableInterrupt();         // Leave nestable interrupt, perform task switch if required
#else
  OS_LeaveInterrupt();                 // Leave interrupt, perform task switch if required
#endif
}

/*********************************************************************
*
*       OS_EnableISR
*/
OS_ISR_HANDLER* OS_EnableISR(int ISRIndex, OS_ISR_HANDLER* pISRHandler) {
  OS_ISR_HANDLER*  pOldHandler;
#if OS_DEBUG
  if ((ISRIndex >= _NUM_INT_SOURCES) || (ISRIndex < 0)) {
    OS_Error(OS_ERR_ISR_INDEX);
    return NULL;
  }
#endif
  OS_DI();
  pOldHandler = _papISR[ISRIndex];       // Get old handler
#if (OS_ISRHANDLERTABLE_ROM == 0)
  _papISR[ISRIndex] = pISRHandler;       // Install new handler
#endif
  OS_RestoreI();
  return pOldHandler;
}

/*********************************************************************
*
*       OS_DisableISR
*/
void OS_DisableISR(int ISRIndex) {
#if OS_DEBUG
  if ((ISRIndex >= _NUM_INT_SOURCES) || (ISRIndex < 0)) {
    OS_Error(OS_ERR_ISR_INDEX);
    return;
  }
#endif
  OS_DI();
#if (OS_ISRHANDLERTABLE_ROM == 0)
  _papISR[ISRIndex] = _OS_IrqDefaultHandler;   // Set default handler
#endif
  OS_RestoreI();
}

/*****  End Of File  ************************************************/

