/*********************************************************************
*                SEGGER Microcontroller GmbH & Co. KG                *
*                        The Embedded Experts                        *
**********************************************************************
*                                                                    *
*       (c) 1995 - 2016 SEGGER Microcontroller GmbH & Co. KG         *
*                                                                    *
*     Internet: segger.com   Support: support_embos@segger.com       *
*                                                                    *
**********************************************************************
*                                                                    *
*       embOS * Real time operating system for microcontrollers      *
*                                                                    *
*                                                                    *
*       Please note:                                                 *
*                                                                    *
*       Knowledge of this file may under no circumstances            *
*       be used to write a similar product or a real-time            *
*       operating system for in-house use.                           *
*                                                                    *
*       Thank you for your fairness !                                *
*                                                                    *
**********************************************************************
*                                                                    *
*       OS version: 4.16                                             *
*                                                                    *
**********************************************************************

----------------------------------------------------------------------
File    : sbrk.c
Purpose : Heap support function for Freescale Coldfire V1 and embOS
--------  END-OF-HEADER  ---------------------------------------------
*/

#include "RTOS.h"

/*********************************************************************
*
*        Global data
*
**********************************************************************
*/
extern char  __HEAP_START[]; 		/* Linker generated symbol */
extern char  __HEAP_END[];  	  /* Linker generated symbol */

/*********************************************************************
*
*        Local data
*
**********************************************************************
*/
static char* __break    = __HEAP_START;
static char* __heap_end = __HEAP_END;

/*********************************************************************
*
*        Global functions
*
**********************************************************************
*/

char *sbrk(int size);

/*********************************************************************
*
*        sbrk()
*
*  Function description:
*    This sbrk() function replaces the one from the run-time library.
*    The original sbrk() function can not be used with embOS.
*    It is called from malloc() an delivers a pointer to a free
*    portion in the heap, if sufficient space is available.
*    The start of the heap and the end of the heap have to be
*    defined in the linker configuration file.
*    The heap grows from start to end.
*
*  Parameter:
*    size = requested size on the heap in bytes
*
*  Return value:
*      -1 : Error, not enough space available
*   != -1 : Pointer to the free area on the heap
*/
char *sbrk(int size) {
  char* oldbreak;
  
  oldbreak = __break;
  if (((__break + size) > __heap_end)) {
    return((char*) -1);
  }
  __break += size;
  return oldbreak;
}

/****** End of File *************************************************/



